@@ -1,5 +1,5 @@
 use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
+use crate::select::SelectedColumns;
 use crate::util;
 use crate::CliResult;
 
@@ -39,11 +39,11 @@ Feeding CSV as stdin (\"-\") to --paths:
     $ cat filelist.csv | xan cat rows --paths - --path-column path > concatenated.csv
 
 Usage:
-    xan cat rows    [options] [<inputs>...]
-    xan cat columns [options] [<inputs>...]
+    xan cat rows [options] [<inputs>...]
+    xan cat (cols|columns) [options] [<inputs>...]
     xan cat --help
 
-cat columns options:
+cat cols/columns options:
     -p, --pad                   When concatenating columns, this flag will cause
                                 all records to appear. It will pad each row if
                                 other CSV data isn't long enough.
@@ -70,9 +70,10 @@ Common options:
 struct Args {
     cmd_rows: bool,
     cmd_columns: bool,
+    cmd_cols: bool,
     arg_inputs: Vec<String>,
     flag_paths: Option<String>,
-    flag_path_column: Option<SelectColumns>,
+    flag_path_column: Option<SelectedColumns>,
     flag_pad: bool,
     flag_output: Option<String>,
     flag_no_headers: bool,
@@ -93,7 +94,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         } else {
             args.cat_rows()
         }
-    } else if args.cmd_columns {
+    } else if args.cmd_columns || args.cmd_cols {
         args.cat_columns()
     } else {
         unreachable!();
@@ -112,22 +113,22 @@ impl Args {
     }
 
     fn cat_rows(&self) -> CliResult<()> {
-        let mut row = csv::ByteRecord::new();
-        let mut wtr = Config::new(&self.flag_output).writer()?;
+        let mut row = simd_csv::ByteRecord::new();
+        let mut wtr = Config::new(&self.flag_output).simd_writer()?;
         for (i, conf) in self.configs()?.into_iter().enumerate() {
-            let mut rdr = conf.reader()?;
+            let mut rdr = conf.simd_reader()?;
 
             match &self.flag_source_column {
                 None => {
-                    if !self.flag_no_headers && i == 0 {
-                        conf.write_headers(&mut rdr, &mut wtr)?;
+                    if !conf.no_headers && i == 0 {
+                        wtr.write_byte_record(rdr.byte_headers()?)?;
                     }
                     while rdr.read_byte_record(&mut row)? {
                         wtr.write_byte_record(&row)?;
                     }
                 }
                 Some(source_column) => {
-                    if !self.flag_no_headers && i == 0 {
+                    if !conf.no_headers && i == 0 {
                         let headers = rdr.byte_headers()?;
                         wtr.write_record([source_column.as_bytes()].into_iter().chain(headers))?;
                     }
@@ -150,8 +151,8 @@ impl Args {
         let paths =
             Config::new(&Some(self.flag_paths.clone().unwrap())).lines(&self.flag_path_column)?;
 
-        let mut record = csv::ByteRecord::new();
-        let mut wtr = Config::new(&self.flag_output).writer()?;
+        let mut record = simd_csv::ByteRecord::new();
+        let mut wtr = Config::new(&self.flag_output).simd_writer()?;
 
         let mut headers_written = self.flag_no_headers;
 
@@ -161,7 +162,7 @@ impl Args {
             let mut reader = Config::new(&Some(path.clone()))
                 .delimiter(self.flag_delimiter)
                 .no_headers(self.flag_no_headers)
-                .reader()?;
+                .simd_reader()?;
 
             match &self.flag_source_column {
                 None => {
@@ -195,7 +196,7 @@ impl Args {
     }
 
     fn cat_columns(&self) -> CliResult<()> {
-        let mut wtr = Config::new(&self.flag_output).writer()?;
+        let mut wtr = Config::new(&self.flag_output).simd_writer()?;
         let mut rdrs = self
             .configs()?
             .into_iter()
@@ -214,7 +215,7 @@ impl Args {
             .map(|rdr| rdr.byte_records())
             .collect::<Vec<_>>();
         'OUTER: loop {
-            let mut record = csv::ByteRecord::new();
+            let mut record = simd_csv::ByteRecord::new();
             let mut num_done = 0;
             for (iter, &len) in iters.iter_mut().zip(lengths.iter()) {
                 match iter.next() {
