@@ -4,7 +4,7 @@ use std::path::PathBuf;
 
 use glob::glob;
 
-static COMMANDS: [&str; 58] = [
+static COMMANDS: [&str; 60] = [
     "agg",
     "behead",
     "bins",
@@ -25,6 +25,7 @@ static COMMANDS: [&str; 58] = [
     "fmt",
     "frequency",
     "from",
+    "fuzzy-join",
     "groupby",
     "guillotine",
     "headers",
@@ -41,11 +42,11 @@ static COMMANDS: [&str; 58] = [
     "network",
     "parallel",
     "partition",
+    "pivot",
     "plot",
     "progress",
     "range",
     "rename",
-    "regex-join",
     "reverse",
     "sample",
     "scrape",
@@ -60,14 +61,17 @@ static COMMANDS: [&str; 58] = [
     "top",
     "transform",
     "transpose",
-    "url-join",
+    "unpivot",
     "view",
     "vocab",
+    "window",
 ];
 
-static HELP_SUBCOMMANDS: [&str; 4] = ["cheatsheet", "functions", "aggs", "scraping"];
+static HELP_SUBCOMMANDS: [&str; 5] = ["cheatsheet", "functions", "aggs", "scraping", "window"];
+static CAT_SUBCOMMANDS: [&str; 2] = ["rows", "columns"];
 static MATRIX_SUBCOMMANDS: [&str; 1] = ["corr"];
 static NETWORK_SUBCOMMANDS: [&str; 2] = ["edgelist", "bipartite"];
+static PARALLEL_SUBCOMMANDS: [&str; 7] = ["count", "cat", "freq", "stats", "agg", "groupby", "map"];
 static SCRAPE_SUBCOMMANDS: [&str; 5] = ["title", "canonical", "links", "urls", "images"];
 static TOKENIZE_SUBCOMMANDS: [&str; 3] = ["words", "sentences", "paragraphs"];
 static VOCAB_SUBCOMMANDS: [&str; 5] = ["corpus", "doc", "doc-token", "token", "cooc"];
@@ -136,6 +140,12 @@ pub fn run() {
                 println!("{}", subcommand);
             }
         }
+    } else if word_before == "cat" && !to_complete.starts_with('-') {
+        for subcommand in CAT_SUBCOMMANDS {
+            if subcommand.starts_with(to_complete) {
+                println!("{}", subcommand);
+            }
+        }
     } else if word_before == "matrix" && !to_complete.starts_with('-') {
         for subcommand in MATRIX_SUBCOMMANDS {
             if subcommand.starts_with(to_complete) {
@@ -148,6 +158,12 @@ pub fn run() {
                 println!("{}", subcommand);
             }
         }
+    } else if word_before == "parallel" && !to_complete.starts_with('-') {
+        for subcommand in PARALLEL_SUBCOMMANDS {
+            if subcommand.starts_with(to_complete) {
+                println!("{}", subcommand);
+            }
+        }
     } else if word_before == "vocab" && !to_complete.starts_with('-') {
         for subcommand in VOCAB_SUBCOMMANDS {
             if subcommand.starts_with(to_complete) {
