@@ -1,14 +1,19 @@
 use std::io::{stdout, Write};
+use std::rc::Rc;
+use std::str;
 
 use ahash::RandomState;
+use colored::Colorize;
 use dlv_list::{Index, VecList};
 use indexmap::{map::Entry as IndexMapEntry, IndexMap};
+use simd_csv::ByteRecord;
 use transient_btree_index::{BtreeConfig, BtreeIndex};
+use unicode_width::UnicodeWidthStr;
 
 use crate::collections::{hash_map::Entry, HashMap, HashSet};
 use crate::config::{Config, Delimiter};
 use crate::moonblade::ChooseProgram;
-use crate::select::SelectColumns;
+use crate::select::SelectedColumns;
 use crate::util;
 use crate::CliResult;
 
@@ -24,6 +29,13 @@ a specific identity to the output, unless you use -l/--keep-last.
 
 The command can also write only the duplicated rows with --keep-duplicates.
 
+You are also given the option to add a column indicating whether each row is
+a duplicate or not, as per selected method, using -f/--flag <name>. You can
+even pipe the result into `xan partition` to split the file into a deduplicated
+one and another containing only discarded duplicates:
+
+    $ xan dedup -f duplicated file.csv | xan partition -s duplicated
+
 Finally, it is also possible to specify which rows to keep by evaluating
 an expression (see `xan help cheatsheet` and `xan help functions` for
 the documentation of the expression language).
@@ -47,24 +59,28 @@ Usage:
     xan dedup --help
 
 dedup options:
-    --check             Verify whether the selection has any duplicates, i.e. whether
-                        the selected columns satisfy a uniqueness constraint.
-    -s, --select <arg>  Select a subset of columns to on which to deduplicate.
-                        See 'xan select --help' for the format details.
-    -S, --sorted        Use if you know your file is already sorted on the deduplication
-                        selection to avoid needing to keep a hashmap of values
-                        in memory.
-    -l, --keep-last     Keep the last row having a specific identity, rather than
-                        the first one. Note that it will cost more memory and that
-                        no rows will be flushed before the whole file has been read
-                        if -S/--sorted is not used.
-    -e, --external      Use an external btree index to keep the index on disk and avoid
-                        overflowing RAM. Does not work with -l/--keep-last and --keep-duplicates.
-    --keep-duplicates   Emit only the duplicated rows.
-    --choose <expr>     Evaluate an expression that must return whether to
-                        keep a newly seen row or not. Column name in the given
-                        expression will be prefixed with \"current_\" for the
-                        currently kept row and \"new_\" for the new row to consider.
+    --check                Verify whether the selection has any duplicates, i.e. whether
+                           the selected columns satisfy a uniqueness constraint.
+    -s, --select <arg>     Select a subset of columns to on which to deduplicate.
+                           See 'xan select --help' for the format details.
+    -S, --sorted           Use if you know your file is already sorted on the deduplication
+                           selection to avoid needing to keep a hashmap of values
+                           in memory.
+    -l, --keep-last        Keep the last row having a specific identity, rather than
+                           the first one. Note that it will cost more memory and that
+                           no rows will be flushed before the whole file has been read
+                           if -S/--sorted is not used.
+    -e, --external         Use an external btree index to keep the index on disk and avoid
+                           overflowing RAM. Does not work with -l/--keep-last and -k/--keep-duplicates.
+    -k, --keep-duplicates  Emit only the duplicated rows.
+    -C, --choose <expr>    Evaluate an expression that must return whether to
+                           keep a newly seen row or not. Column name in the given
+                           expression will be prefixed with \"current_\" for the
+                           currently kept row and \"new_\" for the new row to consider.
+    -f, --flag <name>      Instead of filtering duplicated rows, add a column with given <name>
+                           indicating whether a row is duplicated. File order might get
+                           modified to keep proper performance when -l/--keep-last
+                           or -C/--choose is used.
 
 Common options:
     -h, --help               Display this message
@@ -78,7 +94,7 @@ Common options:
 #[derive(Deserialize)]
 struct Args {
     arg_input: Option<String>,
-    flag_select: SelectColumns,
+    flag_select: SelectedColumns,
     flag_check: bool,
     flag_no_headers: bool,
     flag_output: Option<String>,
@@ -88,10 +104,9 @@ struct Args {
     flag_external: bool,
     flag_keep_duplicates: bool,
     flag_choose: Option<String>,
+    flag_flag: Option<String>,
 }
 
-type DeduplicationKey = Vec<Vec<u8>>;
-
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut args: Args = util::get_args(USAGE, argv)?;
 
@@ -138,23 +153,55 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .no_headers(args.flag_no_headers)
         .select(args.flag_select);
 
-    let mut rdr = rconf.reader()?;
+    let mut rdr = rconf.simd_reader()?;
     let headers = rdr.byte_headers()?.clone();
     let sel = rconf.selection(&headers)?;
 
     if args.flag_check {
-        let mut record = csv::ByteRecord::new();
+        if args.flag_flag.is_some() {
+            Err("-f/--flag does not make sense with --check!")?;
+        }
+
+        let mut already_seen = HashSet::<ByteRecord>::new();
+        let mut record = ByteRecord::new();
+
+        let mut count: u64 = 0;
 
         while rdr.read_byte_record(&mut record)? {
-            let mut already_seen = HashSet::<DeduplicationKey>::new();
+            let key = sel.select(&record).collect();
 
-            while rdr.read_byte_record(&mut record)? {
-                let key = sel.collect(&record);
+            if !already_seen.insert(key) {
+                let max_len_of_head_sel = sel
+                    .select(&headers)
+                    .map(|h| str::from_utf8(h).unwrap().width())
+                    .max()
+                    .unwrap();
 
-                if !already_seen.insert(key) {
-                    Err("selection is NOT unique!")?;
-                }
+                let msg = format!(
+                    "selection is NOT unique!\nFirst duplicate record found at index {}:\n\n{}",
+                    count,
+                    sel.select(&headers)
+                        .map(|h| {
+                            let head_to_print = str::from_utf8(h).unwrap();
+                            format!(
+                                "{}{}",
+                                head_to_print,
+                                " ".repeat(max_len_of_head_sel - head_to_print.width())
+                            )
+                        })
+                        .zip(
+                            sel.select(&record)
+                                .map(|cell| { str::from_utf8(cell).unwrap().red().bold() })
+                        )
+                        .map(|(h, v)| format!("{} {}", h, v))
+                        .collect::<Vec<_>>()
+                        .join("\n"),
+                );
+
+                Err(msg)?;
             }
+
+            count += 1;
         }
 
         writeln!(&mut stdout(), "selection is unique!")?;
@@ -162,13 +209,29 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         return Ok(());
     }
 
-    let mut wtr = Config::new(&args.flag_output).writer()?;
+    let mut wtr = Config::new(&args.flag_output).simd_writer()?;
 
-    rconf.write_headers(&mut rdr, &mut wtr)?;
+    if !rconf.no_headers {
+        if let Some(column_name) = &args.flag_flag {
+            wtr.write_record(headers.iter().chain([column_name.as_bytes()]))?;
+        } else {
+            wtr.write_byte_record(&headers)?;
+        }
+    }
+
+    let mut record_writer = RecordWriter {
+        mode: if args.flag_flag.is_some() {
+            RecordWriterMode::Flag
+        } else {
+            RecordWriterMode::Filter
+        },
+        writer: &mut wtr,
+        inverted: args.flag_keep_duplicates,
+    };
 
     // External
     if args.flag_external {
-        let mut record = csv::ByteRecord::new();
+        let mut record = ByteRecord::new();
 
         let mut btree_index = BtreeIndex::<Vec<Vec<u8>>, ()>::with_capacity(
             BtreeConfig::default().fixed_value_size(0),
@@ -179,7 +242,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             let key = sel.collect(&record);
 
             if btree_index.insert(key, ())?.is_none() {
-                wtr.write_byte_record(&record)?;
+                record_writer.emit_record(&record)?;
+            } else {
+                record_writer.discard_record(&record)?;
             }
         }
 
@@ -206,14 +271,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     match (args.flag_sorted, dedup_mode) {
         // Unsorted, keep first
         (false, DedupMode::KeepFirst) => {
-            let mut record = csv::ByteRecord::new();
-            let mut already_seen = HashSet::<DeduplicationKey>::new();
+            let mut record = ByteRecord::new();
+            let mut already_seen = HashSet::<ByteRecord>::new();
 
             while rdr.read_byte_record(&mut record)? {
-                let key = sel.collect(&record);
+                let key = sel.select(&record).collect();
 
                 if already_seen.insert(key) {
-                    wtr.write_byte_record(&record)?;
+                    record_writer.emit_record(&record)?;
+                } else {
+                    record_writer.discard_record(&record)?;
                 }
             }
         }
@@ -224,105 +291,117 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             for result in rdr.byte_records() {
                 let record = result?;
-                let key = sel.collect(&record);
-                set.push(key, record);
+                let key = sel.select(&record).collect();
+
+                if let Some(discarded_record) = set.push(key, record) {
+                    record_writer.discard_record(&discarded_record)?;
+                }
             }
 
             for record in set.into_iter() {
-                wtr.write_byte_record(&record)?;
+                record_writer.emit_record(&record)?;
             }
         }
 
         // Sorted, keep first
         (true, DedupMode::KeepFirst) => {
-            let mut record = csv::ByteRecord::new();
-            let mut current: Option<DeduplicationKey> = None;
+            let mut record = ByteRecord::new();
+            let mut current: Option<ByteRecord> = None;
 
             while rdr.read_byte_record(&mut record)? {
-                let key = sel.collect(&record);
+                let key = sel.select(&record).collect();
 
                 match current {
                     None => {
-                        wtr.write_byte_record(&record)?;
+                        record_writer.emit_record(&record)?;
                         current = Some(key);
                     }
                     Some(current_key) if current_key != key => {
-                        wtr.write_byte_record(&record)?;
+                        record_writer.emit_record(&record)?;
                         current = Some(key);
                     }
-                    _ => (),
+                    _ => {
+                        record_writer.discard_record(&record)?;
+                    }
                 };
             }
         }
 
         // Sorted, keep last
         (true, DedupMode::KeepLast) => {
-            let mut current: Option<(DeduplicationKey, csv::ByteRecord)> = None;
+            let mut current: Option<(ByteRecord, ByteRecord)> = None;
 
             for result in rdr.byte_records() {
                 let record = result?;
-                let key = sel.collect(&record);
+                let key = sel.select(&record).collect();
 
                 match current {
                     Some((current_key, record_to_flush)) if current_key != key => {
-                        wtr.write_byte_record(&record_to_flush)?;
+                        record_writer.emit_record(&record_to_flush)?;
                     }
-                    _ => (),
+                    Some((_, record_to_flush)) => {
+                        record_writer.discard_record(&record_to_flush)?;
+                    }
+                    _ => {}
                 }
 
                 current = Some((key, record));
             }
 
             if let Some((_, record_to_flush)) = current {
-                wtr.write_byte_record(&record_to_flush)?;
+                record_writer.emit_record(&record_to_flush)?;
             }
         }
 
         // Unsorted, keep duplicates
         (false, DedupMode::KeepDuplicates) => {
-            let mut map: HashMap<DeduplicationKey, Option<(usize, csv::ByteRecord)>> =
-                HashMap::new();
-            let mut rows: Vec<Option<csv::ByteRecord>> = Vec::new();
-            let mut record = csv::ByteRecord::new();
+            let mut map: HashMap<ByteRecord, Option<(usize, Rc<ByteRecord>)>> = HashMap::new();
+            let mut records: Vec<(bool, Rc<ByteRecord>)> = Vec::new();
+            let mut record = ByteRecord::new();
             let mut index: usize = 0;
 
             while rdr.read_byte_record(&mut record)? {
-                let key = sel.collect(&record);
+                let key = sel.select(&record).collect();
+                let record_rc = Rc::new(record.clone());
 
                 match map.entry(key) {
                     Entry::Occupied(mut entry) => {
-                        if let Some((ind, row)) = entry.get_mut().take() {
-                            rows[ind] = Some(row);
+                        if let Some((ind, _)) = entry.get_mut().take() {
+                            records[ind].0 = true;
                         }
-                        rows.push(Some(record.clone()));
+                        records.push((true, record_rc.clone()));
                     }
                     Entry::Vacant(entry) => {
-                        entry.insert(Some((index, record.clone())));
-                        rows.push(None);
+                        entry.insert(Some((index, record_rc.clone())));
+                        records.push((false, record_rc.clone()));
                     }
                 }
                 index += 1;
             }
 
-            for row in rows.into_iter().flatten() {
-                wtr.write_byte_record(&row)?;
+            for (keep, record) in records.into_iter() {
+                if keep {
+                    record_writer.emit_record(&record)?;
+                } else {
+                    record_writer.discard_record(&record)?;
+                }
             }
         }
 
         // Sorted, keep duplicates
         (true, DedupMode::KeepDuplicates) => {
-            let mut record = csv::ByteRecord::new();
+            let mut record = ByteRecord::new();
 
             struct PreviousEntry {
-                key: DeduplicationKey,
-                record: csv::ByteRecord,
+                key: ByteRecord,
+                record: ByteRecord,
                 already_emitted: bool,
             }
 
             let mut previous_entry_opt: Option<PreviousEntry> = None;
 
             while rdr.read_byte_record(&mut record)? {
-                let key = sel.collect(&record);
+                let key = sel.select(&record).collect();
 
                 match previous_entry_opt.as_mut() {
                     None => {
@@ -335,12 +414,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     Some(previous_entry) => {
                         if previous_entry.key == key {
                             if !previous_entry.already_emitted {
-                                wtr.write_byte_record(&previous_entry.record)?;
+                                record_writer.emit_record(&previous_entry.record)?;
                                 previous_entry.already_emitted = true;
                             }
 
-                            wtr.write_byte_record(&record)?;
+                            record_writer.emit_record(&record)?;
                         } else {
+                            if !previous_entry.already_emitted {
+                                record_writer.discard_record(&previous_entry.record)?;
+                            }
+
                             previous_entry_opt = Some(PreviousEntry {
                                 key,
                                 record: record.clone(),
@@ -350,18 +433,24 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     }
                 };
             }
+
+            if let Some(previous_entry) = previous_entry_opt {
+                if !previous_entry.already_emitted {
+                    record_writer.discard_record(&previous_entry.record)?;
+                }
+            }
         }
 
         // Unsorted choose
         (false, DedupMode::Choose(expr)) => {
-            let mut map: IndexMap<DeduplicationKey, csv::ByteRecord, RandomState> =
+            let mut map: IndexMap<ByteRecord, ByteRecord, RandomState> =
                 IndexMap::with_hasher(RandomState::new());
             let mut program = ChooseProgram::parse(&expr, &headers)?;
-            let mut record = csv::ByteRecord::new();
+            let mut record = ByteRecord::new();
             let mut index: usize = 0;
 
             while rdr.read_byte_record(&mut record)? {
-                match map.entry(sel.collect(&record)) {
+                match map.entry(sel.select(&record).collect()) {
                     IndexMapEntry::Vacant(entry) => {
                         entry.insert(record.clone());
                     }
@@ -369,8 +458,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         program.prepare_current_record(entry.get());
 
                         if program.run_with_record(index, &record)? {
+                            record_writer.discard_record(entry.get())?;
                             record.clone_into(entry.get_mut());
-                        };
+                        } else {
+                            record_writer.discard_record(&record)?;
+                        }
                     }
                 }
 
@@ -378,19 +470,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
 
             for output_record in map.into_values() {
-                wtr.write_byte_record(&output_record)?;
+                record_writer.emit_record(&output_record)?;
             }
         }
 
         // Sorted choose
         (true, DedupMode::Choose(expr)) => {
-            let mut current_opt: Option<(DeduplicationKey, csv::ByteRecord)> = None;
+            let mut current_opt: Option<(ByteRecord, ByteRecord)> = None;
             let mut program = ChooseProgram::parse(&expr, &headers)?;
-            let mut record = csv::ByteRecord::new();
+            let mut record = ByteRecord::new();
             let mut index: usize = 0;
 
             while rdr.read_byte_record(&mut record)? {
-                let key = sel.collect(&record);
+                let key = sel.select(&record).collect();
 
                 match current_opt.as_mut() {
                     None => {
@@ -400,13 +492,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     Some((current_key, current_record)) => {
                         if &key == current_key {
                             if program.run_with_record(index, &record)? {
+                                record_writer.discard_record(current_record)?;
                                 // Swap
                                 record.clone_into(current_record);
                                 program.prepare_current_record(current_record);
+                            } else {
+                                record_writer.discard_record(&record)?;
                             }
                         } else {
                             // Flush
-                            wtr.write_byte_record(current_record)?;
+                            record_writer.emit_record(current_record)?;
                             program.prepare_current_record(&record);
                             *current_key = key;
                             record.clone_into(current_record);
@@ -419,7 +514,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             // Flush
             if let Some((_, current_record)) = current_opt {
-                wtr.write_byte_record(&current_record)?;
+                record_writer.emit_record(&current_record)?;
             }
         }
     }
@@ -427,9 +522,66 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     Ok(wtr.flush()?)
 }
 
+enum RecordWriterMode {
+    Filter,
+    Flag,
+}
+
+impl RecordWriterMode {
+    fn is_flag(&self) -> bool {
+        matches!(self, Self::Flag)
+    }
+}
+
+struct RecordWriter<'w, W: Write> {
+    mode: RecordWriterMode,
+    writer: &'w mut simd_csv::Writer<W>,
+    inverted: bool,
+}
+
+impl<W: Write> RecordWriter<'_, W> {
+    #[inline(always)]
+    fn label(&self, mut emit: bool) -> &'static [u8] {
+        if self.inverted {
+            emit = !emit;
+        }
+
+        if emit {
+            b"false"
+        } else {
+            b"true"
+        }
+    }
+
+    fn emit_record(&mut self, record: &ByteRecord) -> CliResult<()> {
+        use RecordWriterMode::*;
+
+        match self.mode {
+            Filter => {
+                self.writer.write_byte_record(record)?;
+            }
+            Flag => {
+                self.writer
+                    .write_record(record.iter().chain([self.label(true)]))?;
+            }
+        };
+
+        Ok(())
+    }
+
+    fn discard_record(&mut self, record: &ByteRecord) -> CliResult<()> {
+        if self.mode.is_flag() {
+            self.writer
+                .write_record(record.iter().chain([self.label(false)]))?;
+        }
+
+        Ok(())
+    }
+}
+
 struct KeepLastSet {
-    map: HashMap<DeduplicationKey, Index<csv::ByteRecord>>,
-    list: VecList<csv::ByteRecord>,
+    map: HashMap<ByteRecord, Index<ByteRecord>>,
+    list: VecList<ByteRecord>,
 }
 
 impl KeepLastSet {
@@ -440,21 +592,24 @@ impl KeepLastSet {
         }
     }
 
-    fn push(&mut self, key: DeduplicationKey, record: csv::ByteRecord) {
+    #[must_use]
+    fn push(&mut self, key: ByteRecord, record: ByteRecord) -> Option<ByteRecord> {
         match self.map.entry(key) {
             Entry::Occupied(mut entry) => {
                 let current_index = entry.get_mut();
 
-                self.list.remove(*current_index);
+                let removed = self.list.remove(*current_index);
                 *current_index = self.list.push_back(record);
+                removed
             }
             Entry::Vacant(entry) => {
                 entry.insert(self.list.push_back(record));
+                None
             }
-        };
+        }
     }
 
-    fn into_iter(self) -> impl Iterator<Item = csv::ByteRecord> {
+    fn into_iter(self) -> impl Iterator<Item = ByteRecord> {
         self.list.into_iter()
     }
 }
