@@ -6,8 +6,8 @@ use regex::{Captures, RegexBuilder};
 use unicode_width::UnicodeWidthStr;
 
 use crate::config::{Config, Delimiter};
-use crate::select::SelectColumns;
-use crate::util;
+use crate::select::SelectedColumns;
+use crate::util::{self, ColorMode};
 use crate::CliResult;
 
 static USAGE: &str = "
@@ -17,10 +17,10 @@ This mode is particularly useful for viewing one record at a time.
 There is also a condensed view (-c or --condense) that will shorten the
 contents of each field to provide a summary view.
 
-Pipe into \"less -r\" if you need to page the result, and use -C/--force-colors
+Pipe into \"less -r\" if you need to page the result, and use --color=always
 not to lose the colors:
 
-    $ xan flatten -C file.csv | less -Sr
+    $ xan flatten --color=always file.csv | less -Sr
 
 Usage:
     xan flatten [options] [<input>]
@@ -34,16 +34,27 @@ flatten options:
     -c, --condense         Don't wrap cell values on new lines but truncate them
                            with ellipsis instead.
     -w, --wrap             Wrap cell values all while minding the header's indent.
+    -F, --flatter          Even flatter representation alternating column name and content
+                           on different lines in the output. Useful to display cells containing
+                           large chunks of text.
+    --row-separator <sep>  Separate rows in the output with the given string, instead of
+                           displaying a header with row index. If an empty string is
+                           given, e.g. --row-separator '', will not separate rows at all.
+    --csv                  Write the result as a CSV file with the row,field,value columns
+                           instead. Can be seen as unpivoting the whole file.
     --cols <num>           Width of the graph in terminal columns, i.e. characters.
                            Defaults to using all your terminal's width or 80 if
                            terminal's size cannot be found (i.e. when piping to file).
                            Can also be given as a ratio of the terminal's width e.g. \"0.5\".
     -R, --rainbow          Alternating colors for cells, rather than color by value type.
-    -C, --force-colors     Force colors even if output is not supposed to be able to
-                           handle them.
+    --color <when>         When to color the output using ANSI escape codes.
+                           Use `auto` for automatic detection, `never` to
+                           disable colors completely and `always` to force
+                           colors, even when the output could not handle them.
+                           [default: auto]
     -S, --split <cols>     Split columns containing multiple values separated by --sep
                            to be displayed as a list.
-    --sep <sep>            Delimiter separating multiple values in cells splitted
+    --sep <sep>            Delimiter separating multiple values in cells split
                            by --plural. [default: |]
     -H, --highlight <pat>  Highlight in red parts of text cells matching given regex
                            pattern. Will not work with -R/--rainbow.
@@ -51,6 +62,8 @@ flatten options:
 
 Common options:
     -h, --help             Display this message
+    -o, --output <file>    Write output to <file> instead of stdout. Only used
+                           when --csv is set.
     -n, --no-headers       When set, the first row will not be interpreted
                            as headers. When set, the name of each field
                            will be its index.
@@ -61,30 +74,37 @@ Common options:
 #[derive(Deserialize)]
 struct Args {
     arg_input: Option<String>,
-    flag_select: SelectColumns,
+    flag_select: SelectedColumns,
     flag_limit: Option<NonZeroUsize>,
     flag_condense: bool,
     flag_wrap: bool,
+    flag_flatter: bool,
+    flag_row_separator: Option<String>,
     flag_cols: Option<String>,
     flag_rainbow: bool,
-    flag_force_colors: bool,
-    flag_split: Option<SelectColumns>,
+    flag_csv: bool,
+    flag_color: ColorMode,
+    flag_split: Option<SelectedColumns>,
     flag_sep: String,
     flag_highlight: Option<String>,
     flag_ignore_case: bool,
     flag_no_headers: bool,
     flag_delimiter: Option<Delimiter>,
+    flag_output: Option<String>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
+    args.flag_color.apply();
 
     if args.flag_rainbow && args.flag_highlight.is_some() {
         Err("-R/--rainbow does not work with -H/--highlight!")?;
     }
 
-    if args.flag_wrap && args.flag_condense {
-        Err("-w/--wrap does not work with -c/--condense!")?;
+    let modalities = args.flag_wrap as u8 + args.flag_condense as u8 + args.flag_flatter as u8;
+
+    if modalities > 1 {
+        Err("must choose only one of -w/--wrap, -c/--condense or -F/--flatter!")?;
     }
 
     let output = io::stdout();
@@ -93,6 +113,47 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers)
         .select(args.flag_select.clone());
+
+    let mut record_index: usize = 0;
+
+    if args.flag_csv {
+        let mut rdr = rconfig.simd_reader()?;
+        let byte_headers = rdr.byte_headers()?.clone();
+        let sel = rconfig.selection(&byte_headers)?;
+
+        let mut wtr = Config::new(&args.flag_output).simd_writer()?;
+
+        let mut output_record = simd_csv::ByteRecord::new();
+        output_record.push_field(b"row");
+        output_record.push_field(b"field");
+        output_record.push_field(b"value");
+
+        wtr.write_byte_record(&output_record)?;
+
+        let mut record = simd_csv::ByteRecord::new();
+
+        while rdr.read_byte_record(&mut record)? {
+            for (h, cell) in sel.select(&byte_headers).zip(sel.select(&record)) {
+                output_record.clear();
+                output_record.push_field(record_index.to_string().as_bytes());
+                output_record.push_field(h);
+                output_record.push_field(cell);
+
+                wtr.write_byte_record(&output_record)?;
+            }
+
+            record_index += 1;
+
+            if let Some(limit) = args.flag_limit {
+                if record_index >= limit.get() {
+                    break;
+                }
+            }
+        }
+
+        return Ok(wtr.flush()?);
+    }
+
     let mut rdr = rconfig.reader()?;
     let byte_headers = rdr.byte_headers()?;
     let sel = rconfig.selection(byte_headers)?;
@@ -102,7 +163,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .map(|cols| {
             cols.selection(
                 &sel.select(byte_headers).collect::<csv::ByteRecord>(),
-                !args.flag_no_headers,
+                !rconfig.no_headers,
             )
         })
         .transpose()?;
@@ -117,10 +178,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         })
         .transpose()?;
 
-    if args.flag_force_colors {
-        colored::control::set_override(true);
-    }
-
     let cols = util::acquire_term_cols_ratio(&args.flag_cols)?;
 
     let potential_headers = rdr.headers()?.clone();
@@ -137,6 +194,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         headers.push(header);
     }
 
+    headers = headers
+        .into_iter()
+        .map(|name| util::sanitize_text_for_single_line_printing(&name))
+        .collect();
+
     let max_header_width = headers
         .iter()
         .map(|h| h.width())
@@ -148,7 +210,6 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
 
     let mut record = csv::StringRecord::new();
-    let mut record_index: usize = 0;
 
     let max_value_width = cols - max_header_width - 1;
 
@@ -179,42 +240,66 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 max_header_width + 1 + offset,
             )
         } else {
-            util::highlight_trimmable_whitespace(cell)
+            cell.to_string()
         };
 
-        let cell = util::colorize(&cell_colorizer, &cell);
-
-        match (cell_colorizer.is_green(), &highlight_pattern) {
-            (true, Some(pattern)) => pattern
-                .replace_all(&cell.to_string(), |caps: &Captures| {
+        let mut cell = match (cell_colorizer.highlightable_color(), &highlight_pattern) {
+            (Some(fg), Some(pattern)) => pattern
+                .replace_all(&cell, |caps: &Captures| {
                     let mut r = String::from("\x1b[0;1;31m");
                     r.push_str(&caps[0]);
-                    r.push_str("\x1b[0;32m");
+                    r.push_str("\x1b[0;");
+                    r.push_str(&fg);
+                    r.push('m');
                     r
                 })
                 .into_owned(),
-            _ => cell.to_string(),
+            _ => cell,
+        };
+
+        if !args.flag_condense {
+            cell = util::highlight_problematic_string_features(&cell);
         }
+
+        util::colorize(&cell_colorizer, &cell).to_string()
     };
 
-    while rdr.read_record(&mut record)? {
-        let record = sel.select(&record).collect::<csv::StringRecord>();
-        if record_index > 0 {
-            writeln!(&output)?;
-        }
-        writeln!(&output, "{}", format!("Row n°{}", record_index).bold())?;
-        writeln!(&output, "{}", "─".repeat(cols).dimmed())?;
+    let display_headers = headers
+        .iter()
+        .map(|header| {
+            util::unicode_aware_highlighted_pad_with_ellipsis(
+                false,
+                header,
+                max_header_width + 1,
+                " ",
+                true,
+            )
+        })
+        .collect::<Vec<_>>();
 
-        for (i, (header, cell)) in headers.iter().zip(record.iter()).enumerate() {
+    while rdr.read_record(&mut record)? {
+        if record_index > 0 {
+            if let Some(separator) = &args.flag_row_separator {
+                if !separator.is_empty() {
+                    writeln!(&output, "{}", &separator)?;
+                }
+            } else {
+                writeln!(&output)?;
+            }
+        }
+
+        if args.flag_row_separator.is_none() {
+            writeln!(&output, "{}", format!("Row n°{}", record_index).bold())?;
+            writeln!(&output, "{}", "─".repeat(cols).dimmed())?;
+        }
+
+        for (i, (header, cell)) in display_headers.iter().zip(sel.select(&record)).enumerate() {
+            // Split cell
             if matches!(&split_sel_opt, Some(split_sel) if !cell.is_empty() && split_sel.contains(i))
             {
                 let mut first: bool = true;
 
-                write!(
-                    &output,
-                    "{}",
-                    util::unicode_aware_rpad(header, max_header_width + 1, " ")
-                )?;
+                write!(&output, "{}", header)?;
 
                 for sub_cell in cell.split(&args.flag_sep) {
                     let sub_cell = prepare_cell(i, sub_cell, 2);
@@ -237,14 +322,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 continue;
             }
 
+            // Regular cell
             let cell = prepare_cell(i, cell, 0);
 
-            writeln!(
-                &output,
-                "{}{}",
-                util::unicode_aware_rpad(header, max_header_width + 1, " "),
-                cell
-            )?;
+            if args.flag_flatter {
+                writeln!(&output, "{}", header)?;
+                writeln!(&output, "{}\n", cell)?;
+            } else {
+                writeln!(&output, "{}{}", header, cell)?;
+            }
         }
 
         record_index += 1;
