@@ -2,16 +2,18 @@ use std::convert::TryFrom;
 use std::num::NonZeroUsize;
 use std::{
     fs,
-    io::{self, BufRead, BufReader, Cursor, Read},
+    io::{self, BufReader, Cursor, Read},
     path::Path,
 };
 
 use calamine::{open_workbook_auto_from_rs, Data, Reader};
 use flate2::read::MultiGzDecoder;
+use jiff::civil::{DateTime, Time};
 use serde_json::{Map, Value};
+use simd_csv::ByteRecord;
 
 use crate::config::Config;
-use crate::json::for_each_json_value_as_csv_record;
+use crate::json::JSONTabularizer;
 use crate::util::{self, ChunksIteratorExt};
 use crate::CliError;
 use crate::CliResult;
@@ -83,7 +85,7 @@ Some formats can be streamed, some others require the full file to be loaded int
 memory. The streamable formats are `ndjson`, `jsonl`, `tar`, `txt` and `npy`.
 
 Some formats will handle gzip decompression on the fly if the filename ends
-in `.gz`: `json`, `ndjson`, `jsonl` and `txt`.
+in `.gz`: `json`, `ndjson`, `jsonl`, `tar` and `txt`.
 
 Tarball extraction was designed for utf8-encoded text files. Expect weird or
 broken results with other encodings or binary files.
@@ -140,8 +142,8 @@ struct Args {
 }
 
 impl Args {
-    fn writer(&self) -> io::Result<csv::Writer<Box<dyn io::Write + Send>>> {
-        Config::new(&self.flag_output).writer()
+    fn writer(&self) -> io::Result<simd_csv::Writer<Box<dyn io::Write + Send>>> {
+        Config::new(&self.flag_output).simd_writer()
     }
 
     fn convert_xls(&self) -> CliResult<()> {
@@ -171,7 +173,7 @@ impl Args {
         }
 
         let mut wtr = self.writer()?;
-        let mut record = csv::StringRecord::new();
+        let mut record = ByteRecord::new();
 
         let range = match &self.flag_sheet_name {
             Some(name) => Some(workbook.worksheet_range(name)),
@@ -202,17 +204,37 @@ impl Args {
 
                     for cell in row {
                         match cell {
-                            Data::String(value) => record.push_field(value),
-                            Data::DateTimeIso(value) => record.push_field(value),
-                            Data::DurationIso(value) => record.push_field(value),
+                            Data::String(value) => record.push_field(value.as_bytes()),
+                            Data::DateTimeIso(value) => record.push_field(value.as_bytes()),
+                            Data::DurationIso(value) => record.push_field(value.as_bytes()),
                             Data::Bool(value) => {
-                                record.push_field(if *value { "true" } else { "false" })
+                                record.push_field(if *value { b"true" } else { b"false" })
                             }
-                            Data::Int(value) => record.push_field(&value.to_string()),
-                            Data::Float(value) => record.push_field(&value.to_string()),
-                            Data::DateTime(value) => record.push_field(&value.to_string()),
-                            Data::Error(err) => record.push_field(&err.to_string()),
-                            Data::Empty => record.push_field(""),
+                            Data::Int(value) => record.push_field(value.to_string().as_bytes()),
+                            Data::Float(value) => record.push_field(value.to_string().as_bytes()),
+                            Data::DateTime(value) => {
+                                let (year, month, day, hour, minute, second, millisecond) =
+                                    value.to_ymd_hms_milli();
+
+                                let datetime = DateTime::new(
+                                    year as i16,
+                                    month as i8,
+                                    day as i8,
+                                    hour as i8,
+                                    minute as i8,
+                                    second as i8,
+                                    millisecond as i32 * 1_000_000,
+                                )
+                                .unwrap();
+
+                                if datetime.time() == Time::MIN {
+                                    record.push_field(datetime.date().to_string().as_bytes());
+                                } else {
+                                    record.push_field(datetime.to_string().as_bytes());
+                                }
+                            }
+                            Data::Error(err) => record.push_field(err.to_string().as_bytes()),
+                            Data::Empty => record.push_field(b""),
                         }
                     }
 
@@ -225,31 +247,32 @@ impl Args {
     }
 
     fn convert_ndjson(&self) -> CliResult<()> {
-        let mut wtr = self.writer()?;
-        let rdr = BufReader::new(Config::new(&self.arg_input).io_reader()?);
+        use simd_json::Buffers;
 
-        for_each_json_value_as_csv_record(
-            rdr.lines().map(|line| -> Result<Value, CliError> {
-                serde_json::from_str(&line?).map_err(|err| CliError::Other(err.to_string()))
-            }),
-            self.flag_sample_size,
-            |record| -> CliResult<()> {
-                wtr.write_record(record)?;
-                Ok(())
-            },
-        )?;
+        let mut buffers = Buffers::default();
 
-        Ok(wtr.flush()?)
+        let wtr = self.writer()?;
+        let mut rdr = simd_csv::LineReader::new(Config::new(&self.arg_input).io_reader()?);
+        let mut tabularizer = JSONTabularizer::from_writer(wtr, self.flag_sample_size);
+
+        while let Some(line) = rdr.read_line()? {
+            // Sshhh... it's alright, really.
+            let line_mut =
+                unsafe { std::slice::from_raw_parts_mut(line.as_ptr() as *mut u8, line.len()) };
+
+            let value: Value = simd_json::serde::from_slice_with_buffers(line_mut, &mut buffers)?;
+
+            tabularizer.process(value)?;
+        }
+
+        Ok(tabularizer.flush()?)
     }
 
     fn convert_json(&self) -> CliResult<()> {
-        let mut rdr = Config::new(&self.arg_input).io_reader()?;
-
-        let mut contents = String::new();
-        rdr.read_to_string(&mut contents)?;
+        let rdr = BufReader::new(Config::new(&self.arg_input).io_reader()?);
 
         let mut value =
-            serde_json::from_str(&contents).map_err(|err| CliError::Other(err.to_string()))?;
+            serde_json::from_reader(rdr).map_err(|err| CliError::Other(err.to_string()))?;
 
         // NOTE: recombobulating objects as collections
         if let Value::Object(object) = value {
@@ -266,18 +289,14 @@ impl Args {
         }
 
         if let Value::Array(array) = value {
-            let mut wtr = self.writer()?;
+            let wtr = self.writer()?;
+            let mut tabularizer = JSONTabularizer::from_writer(wtr, self.flag_sample_size);
 
-            for_each_json_value_as_csv_record(
-                array.into_iter().map(Ok),
-                self.flag_sample_size,
-                |record| -> CliResult<()> {
-                    wtr.write_record(record)?;
-                    Ok(())
-                },
-            )?;
+            for item in array.into_iter() {
+                tabularizer.process(item)?;
+            }
 
-            Ok(wtr.flush()?)
+            Ok(tabularizer.flush()?)
         } else {
             Err(CliError::Other(
                 "target JSON does not contain an array nor an object".to_string(),
@@ -286,24 +305,13 @@ impl Args {
     }
 
     fn convert_text_lines(&self) -> CliResult<()> {
-        let rdr = BufReader::new(Config::new(&self.arg_input).io_reader()?);
+        let mut rdr = simd_csv::LineReader::new(Config::new(&self.arg_input).io_reader()?);
         let mut wtr = self.writer()?;
+
         wtr.write_record([&self.flag_column])?;
 
-        let mut record = csv::ByteRecord::new();
-
-        for result in rdr.lines() {
-            let line = result?;
-            let line = line.trim_end();
-
-            if line.is_empty() {
-                continue;
-            }
-
-            record.clear();
-            record.push_field(line.as_bytes());
-
-            wtr.write_byte_record(&record)?;
+        while let Some(line) = rdr.read_line()? {
+            wtr.write_record([line])?;
         }
 
         Ok(wtr.flush()?)
@@ -336,13 +344,7 @@ impl Args {
 
         let mut wtr = self.writer()?;
 
-        let mut record = csv::ByteRecord::new();
-
-        for i in 0..columns {
-            record.push_field(format!("dim_{}", i).as_bytes());
-        }
-
-        wtr.write_byte_record(&record)?;
+        wtr.write_record_no_quoting((0..columns).map(|i| format!("dim_{}", i)))?;
 
         macro_rules! process {
             ($type: ty) => {
@@ -351,13 +353,10 @@ impl Args {
                     .unwrap()
                     .chunks(NonZeroUsize::new(columns as usize).unwrap())
                 {
-                    record.clear();
-
-                    for cell in row {
-                        record.push_field(cell.unwrap().to_string().as_bytes());
-                    }
-
-                    wtr.write_byte_record(&record)?;
+                    // NOTE: this is safe because the output delimiter is guaranteed to be a comma
+                    wtr.write_record_no_quoting(
+                        row.into_iter().map(|cell| cell.unwrap().to_string()),
+                    )?;
                 }
             };
         }
@@ -376,24 +375,22 @@ impl Args {
     }
 
     fn convert_tar(&self) -> CliResult<()> {
-        let rdr: Box<dyn Read> = match self.arg_input.as_ref() {
+        let mut rdr: Box<dyn Read> = match self.arg_input.as_ref() {
             None => Box::new(io::stdin()),
             Some(p) => Box::new(fs::File::open(p)?),
         };
 
-        let rdr = MultiGzDecoder::new(rdr);
+        if matches!(self.arg_input.as_ref(), Some(p) if p.ends_with(".gz")) {
+            rdr = Box::new(MultiGzDecoder::new(rdr));
+        }
+
         let mut archive = tar::Archive::new(rdr);
 
         let mut wtr = self.writer()?;
 
-        let mut record = csv::ByteRecord::new();
         let mut bytes: Vec<u8> = Vec::new();
 
-        record.push_field(b"path");
-        record.push_field(b"size");
-        record.push_field(b"content");
-
-        wtr.write_byte_record(&record)?;
+        wtr.write_record(["path".as_bytes(), b"size", b"content"])?;
 
         for result in archive.entries()? {
             let mut entry = result?;
@@ -411,12 +408,11 @@ impl Args {
                 entry.read_to_end(&mut bytes)?;
             }
 
-            record.clear();
-            record.push_field(&entry.path_bytes());
-            record.push_field(entry.size().to_string().as_bytes());
-            record.push_field(&bytes);
-
-            wtr.write_byte_record(&record)?;
+            wtr.write_record([
+                &entry.path_bytes(),
+                entry.size().to_string().as_bytes(),
+                &bytes,
+            ])?;
         }
 
         Ok(wtr.flush()?)
@@ -465,7 +461,7 @@ impl Args {
             })?;
 
         let mut wtr = self.writer()?;
-        let mut record = csv::ByteRecord::new();
+        let mut record = ByteRecord::new();
         let lines = buf.lines().collect::<Vec<_>>();
         for row in table.children() {
             // Ignore non-row nodes, though there shouldn't be any.
