@@ -9,9 +9,9 @@ use unicode_width::UnicodeWidthStr;
 
 use crate::config::{Config, Delimiter};
 use crate::dates;
-use crate::scales::LinearScale;
-use crate::select::SelectColumns;
-use crate::util;
+use crate::scales::{Scale, ScaleType};
+use crate::select::SelectedColumns;
+use crate::util::{self, ColorMode};
 use crate::CliResult;
 
 const SMALL_BAR_CHARS: [&str; 2] = ["╸", "━"]; // "╾"
@@ -56,14 +56,20 @@ hist options:
     -c, --category <col>     Name of the categorical column that will be used to
                              assign distinct colors per category.
                              Incompatible with -R, --rainbow.
-    -C, --force-colors       Force colors even if output is not supposed to be able to
-                             handle them.
+    --color <when>           When to color the output using ANSI escape codes.
+                             Use `auto` for automatic detection, `never` to
+                             disable colors completely and `always` to force
+                             colors, even when the output could not handle them.
+                             [default: auto]
     -P, --hide-percent       Don't show percentages.
     -u, --unit <unit>        Value unit.
     -D, --dates              Set to indicate your values are dates (supporting year, year-month or
                              year-month-day). This will sort the bars by date, and add missing dates.
     -G, --compress-gaps <n>  If given, will compress gaps of minimum <n> consecutive
                              entries set to 0 and replace it with an ellipsis.
+    --scale <scale>          Apply a scale to the values. Can be one of \"lin\", \"log\",
+                             \"log2\", \"log10\" or \"log(custom_base)\" like \"log(2.5)\".
+                             [default: lin]
 
 Common options:
     -h, --help             Display this message
@@ -78,32 +84,30 @@ struct Args {
     arg_input: Option<String>,
     flag_no_headers: bool,
     flag_delimiter: Option<Delimiter>,
-    flag_field: SelectColumns,
-    flag_label: SelectColumns,
-    flag_value: SelectColumns,
+    flag_field: SelectedColumns,
+    flag_label: SelectedColumns,
+    flag_value: SelectedColumns,
     flag_cols: Option<String>,
-    flag_force_colors: bool,
+    flag_color: ColorMode,
     flag_domain_max: String,
     flag_rainbow: bool,
     flag_name: String,
     flag_hide_percent: bool,
     flag_unit: Option<String>,
-    flag_category: Option<SelectColumns>,
+    flag_category: Option<SelectedColumns>,
     flag_bar_size: String,
     flag_dates: bool,
     flag_compress_gaps: Option<usize>,
+    flag_scale: ScaleType,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
     let args: Args = util::get_args(USAGE, argv)?;
+    args.flag_color.apply();
     let conf = Config::new(&args.arg_input)
         .delimiter(args.flag_delimiter)
         .no_headers(args.flag_no_headers);
 
-    if args.flag_force_colors {
-        colored::control::set_override(true);
-    }
-
     if args.flag_category.is_some() && args.flag_rainbow {
         Err("-c, --category cannot work with -R, --rainbow")?;
     }
@@ -115,13 +119,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     let label_pos = args
         .flag_label
-        .single_selection(&headers, !args.flag_no_headers)?;
+        .single_selection(&headers, !conf.no_headers)?;
     let value_pos = args
         .flag_value
-        .single_selection(&headers, !args.flag_no_headers)?;
+        .single_selection(&headers, !conf.no_headers)?;
     let field_pos_option = args
         .flag_field
-        .single_selection(&headers, !args.flag_no_headers)
+        .single_selection(&headers, !conf.no_headers)
         .ok();
 
     let mut histograms = Histograms::new();
@@ -131,7 +135,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let category_column_index = args
         .flag_category
         .as_ref()
-        .map(|name| name.single_selection(&headers, !args.flag_no_headers))
+        .map(|name| name.single_selection(&headers, !conf.no_headers))
         .transpose()?;
 
     let mut category_colors: IndexMap<String, usize, RandomState> =
@@ -148,6 +152,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             .parse::<f64>()
             .map_err(|_| "could not parse value")?;
 
+        if args.flag_scale.is_logarithmic() && (value < 0.0 || (value > 0.0 && value < 1.0)) {
+            Err(format!(
+                "{} value cannot work with a log scale!",
+                value.to_string().cyan()
+            ))?;
+        }
+
         if let Some(category_col) = category_column_index {
             let category = record[category_col].to_string();
 
@@ -214,13 +225,18 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
         writeln!(
             &mut out,
-            "\nHistogram for {} (bars: {}, sum: {}{}, max: {}{}):\n",
+            "\nHistogram for {} (bars: {}, sum: {}{}, max: {}{}){}:\n",
             histogram.field.green(),
             util::format_number(histogram.len()).cyan(),
             util::format_number(sum).cyan(),
             unit.cyan(),
             util::format_number(histogram.max().unwrap()).cyan(),
             unit.cyan(),
+            if args.flag_scale.is_linear() {
+                "".to_string()
+            } else {
+                format!(", {} scale", args.flag_scale.to_string().cyan())
+            }
         )?;
 
         let pct_cols: usize = if args.flag_hide_percent { 0 } else { 8 };
@@ -252,7 +268,18 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             _ => Err("unknown -B, --bar-size. Should be one of \"small\", \"medium\", \"large\".")?,
         };
 
-        let scale = LinearScale::new((0.0, domain_max), (0.0, bar_cols as f64));
+        let scale = Scale::new(
+            args.flag_scale,
+            (
+                if args.flag_scale.is_logarithmic() {
+                    1.0
+                } else {
+                    0.0
+                },
+                domain_max,
+            ),
+            (0.0, bar_cols as f64),
+        );
 
         for (i, bar_opt) in histogram
             .compressed_bars(args.flag_compress_gaps)
@@ -266,7 +293,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             let bar = bar_opt.unwrap();
 
-            let bar_width = scale.map(bar.value);
+            let bar_width = if scale.is_logarithmic() && bar.value == 0.0 {
+                0.0
+            } else {
+                scale.map(bar.value)
+            };
 
             let mut bar_as_chars =
                 util::unicode_aware_rpad(&create_bar(chars, bar_width), bar_cols, " ").clear();
@@ -305,16 +336,21 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 _ => label.normal(),
             };
 
+            let count = util::unicode_aware_lpad_with_ellipsis(
+                &util::format_number(bar.value),
+                count_cols,
+                " ",
+            );
+
             writeln!(
                 &mut out,
                 "{} |{}{}{}|{}|",
                 label,
-                util::unicode_aware_lpad_with_ellipsis(
-                    &util::format_number(bar.value),
-                    count_cols,
-                    " "
-                )
-                .cyan(),
+                if bar.value == 0.0 {
+                    count.dimmed()
+                } else {
+                    count.cyan()
+                },
                 unit.cyan(),
                 if args.flag_hide_percent {
                     "".to_string().normal()
@@ -522,6 +558,12 @@ impl Histogram {
                     previous_date_opt = Some(date);
                 }
                 Some(mut previous_date) => {
+                    if date == previous_date {
+                        self.bars.last_mut().unwrap().value += bar.value;
+
+                        continue;
+                    }
+
                     loop {
                         let expected_date = dates::next_partial_date(unit, &previous_date);
 
