@@ -3,7 +3,6 @@ use std::io::{copy, Read, SeekFrom};
 
 use crate::config::{Config, Delimiter};
 use crate::moonblade::Program;
-use crate::read::read_byte_record_up_to;
 use crate::util;
 use crate::CliResult;
 
@@ -20,7 +19,7 @@ struct Conditions {
 }
 
 impl Conditions {
-    fn process(&mut self, index: usize, record: &csv::ByteRecord) -> CliResult<ControlFlow> {
+    fn process(&mut self, index: usize, record: &simd_csv::ByteRecord) -> CliResult<ControlFlow> {
         if !self.has_started {
             if let Some(program) = &self.start {
                 let value = program.run_with_record(index, record)?;
@@ -173,6 +172,12 @@ impl Args {
     pub fn run(mut self) -> CliResult<()> {
         self.resolve();
 
+        if let (Some(start_byte), Some(end_byte)) = (self.flag_byte_offset, self.flag_end_byte) {
+            if end_byte <= start_byte {
+                Err("--B/--byte-offset must be less than --end-byte!")?;
+            }
+        }
+
         if self.flag_raw {
             if self.flag_byte_offset.is_none() || self.flag_end_byte.is_none() {
                 Err("--raw requires both -B/--byte-offset & --end-byte!")?;
@@ -183,7 +188,7 @@ impl Args {
 
             let mut wtr = wconf.io_writer()?;
 
-            let mut rdr = rconf.io_reader_for_random_access()?;
+            let mut rdr = rconf.seekable_io_reader()?;
 
             if !rconf.no_headers {
                 let mut csv_rdr = rconf.csv_reader_from_reader(&mut rdr);
@@ -215,55 +220,89 @@ impl Args {
             }
 
             return {
-                let rconf = self.rconfig();
+                let mut rconf = self.rconfig();
+                let no_headers = rconf.no_headers;
 
                 if let Some(offset) = self.flag_byte_offset {
-                    let inner = rconf.io_reader_for_random_access()?;
-                    let mut rdr = rconf.csv_reader_from_reader(inner);
+                    rconf = rconf.no_headers(true);
+                    let mut rdr = rconf.simd_csv_reader_from_reader(rconf.seekable_io_reader()?);
+                    let headers = rdr.byte_headers()?.clone();
+                    let mut inner = rdr.into_inner();
 
-                    let mut pos = csv::Position::new();
-                    pos.set_byte(offset);
+                    inner.seek(SeekFrom::Start(offset))?;
 
-                    rdr.seek_raw(SeekFrom::Start(offset), pos)?;
-
-                    self.run_plural(rdr)
+                    if let Some(end_offset) = self.flag_end_byte {
+                        self.run_plural(
+                            no_headers,
+                            rconf.simd_csv_reader_from_reader(inner.take(end_offset - offset)),
+                            headers,
+                        )
+                    } else {
+                        self.run_plural(
+                            no_headers,
+                            rconf.simd_csv_reader_from_reader(inner),
+                            headers,
+                        )
+                    }
                 } else {
-                    let rdr = rconf.reader()?;
-                    self.run_plural(rdr)
+                    let mut rdr = rconf.simd_reader()?;
+                    let headers = rdr.byte_headers()?.clone();
+                    self.run_plural(no_headers, rdr, headers)
                 }
             };
         }
 
-        let rconf = self.rconfig();
+        let mut rconf = self.rconfig();
+        let no_headers = rconf.no_headers;
 
         if let Some(offset) = self.flag_byte_offset {
-            let inner = rconf.io_reader_for_random_access()?;
-            let mut rdr = rconf.csv_reader_from_reader(inner);
+            rconf = rconf.no_headers(true);
+            let mut rdr = rconf.simd_csv_reader_from_reader(rconf.seekable_io_reader()?);
+            let headers = rdr.byte_headers()?.clone();
+            let mut inner = rdr.into_inner();
 
-            let mut pos = csv::Position::new();
-            pos.set_byte(offset);
+            inner.seek(SeekFrom::Start(offset))?;
 
-            rdr.seek_raw(SeekFrom::Start(offset), pos)?;
-
-            self.run_default(rdr)
+            if let Some(end_offset) = self.flag_end_byte {
+                self.run_default(
+                    no_headers,
+                    rconf.simd_csv_reader_from_reader(inner.take(end_offset - offset)),
+                    headers,
+                )
+            } else {
+                self.run_default(
+                    no_headers,
+                    rconf.simd_csv_reader_from_reader(inner),
+                    headers,
+                )
+            }
         } else {
-            let rdr = rconf.reader()?;
-            self.run_default(rdr)
+            let mut rdr = rconf.simd_reader()?;
+            let headers = rdr.byte_headers()?.clone();
+            self.run_default(no_headers, rdr, headers)
         }
     }
 
-    fn run_default<R: Read>(&self, mut rdr: csv::Reader<R>) -> CliResult<()> {
-        let mut wtr = self.wconfig().writer()?;
-        self.rconfig().write_headers(&mut rdr, &mut wtr)?;
+    fn run_default<R: Read>(
+        &self,
+        no_headers: bool,
+        mut rdr: simd_csv::Reader<R>,
+        headers: simd_csv::ByteRecord,
+    ) -> CliResult<()> {
+        let mut wtr = self.wconfig().simd_writer()?;
 
-        let mut record = csv::ByteRecord::new();
-        let mut conditions = self.conditions(rdr.byte_headers()?)?;
+        if !no_headers {
+            wtr.write_byte_record(&headers)?;
+        }
+
+        let mut record = simd_csv::ByteRecord::new();
+        let mut conditions = self.conditions(&headers)?;
 
         let (start, end) = self.range()?;
         let mut record_index: usize = 0;
         let mut i: usize = 0;
 
-        while read_byte_record_up_to(&mut rdr, &mut record, self.flag_end_byte)? {
+        while rdr.read_byte_record(&mut record)? {
             match conditions.process(record_index, &record)? {
                 ControlFlow::Break => break,
                 ControlFlow::Continue => continue,
@@ -290,14 +329,15 @@ impl Args {
 
     fn run_last(&self) -> CliResult<()> {
         let rconf = self.rconfig();
-        let mut wtr = self.wconfig().writer()?;
 
         let n = self.flag_last.unwrap();
 
         match rconf.reverse_reader() {
-            Ok((headers, mut reverse_reader)) => {
-                if !self.flag_no_headers {
-                    wtr.write_byte_record(&headers)?;
+            Ok(mut reverse_reader) => {
+                let mut wtr = self.wconfig().simd_writer()?;
+
+                if !rconf.no_headers {
+                    wtr.write_byte_record(reverse_reader.byte_headers())?;
                 }
 
                 let records = reverse_reader
@@ -306,26 +346,24 @@ impl Args {
                     .collect::<Result<Vec<_>, _>>()?;
 
                 for record in records.into_iter().rev() {
-                    wtr.write_record(
-                        record
-                            .iter()
-                            .rev()
-                            .map(|cell| cell.iter().rev().copied().collect::<Vec<_>>()),
-                    )?;
+                    wtr.write_byte_record(&record)?;
                 }
+
+                Ok(wtr.flush()?)
             }
             Err(_) => {
-                let mut rdr = rconf.reader()?;
+                let mut rdr = rconf.simd_reader()?;
+                let mut wtr = self.wconfig().simd_writer()?;
 
                 let n = self.flag_last.unwrap();
 
                 let headers = rdr.byte_headers()?.clone();
 
-                if !self.flag_no_headers {
+                if !rconf.no_headers {
                     wtr.write_byte_record(&headers)?;
                 }
 
-                let mut buffer: VecDeque<csv::ByteRecord> = VecDeque::with_capacity(n);
+                let mut buffer: VecDeque<simd_csv::ByteRecord> = VecDeque::with_capacity(n);
 
                 for result in rdr.byte_records() {
                     if buffer.len() >= n {
@@ -338,22 +376,30 @@ impl Args {
                 for record in buffer {
                     wtr.write_byte_record(&record)?;
                 }
-            }
-        };
 
-        Ok(wtr.flush()?)
+                Ok(wtr.flush()?)
+            }
+        }
     }
 
-    fn run_plural<R: Read>(&self, mut rdr: csv::Reader<R>) -> CliResult<()> {
-        let mut wtr = self.wconfig().writer()?;
-        self.rconfig().write_headers(&mut rdr, &mut wtr)?;
+    fn run_plural<R: Read>(
+        &self,
+        no_headers: bool,
+        mut rdr: simd_csv::Reader<R>,
+        headers: simd_csv::ByteRecord,
+    ) -> CliResult<()> {
+        let mut wtr = self.wconfig().simd_writer()?;
+
+        if !no_headers {
+            wtr.write_byte_record(&headers)?;
+        }
 
         let indices = self.plural_indices()?;
 
-        let mut record = csv::ByteRecord::new();
+        let mut record = simd_csv::ByteRecord::new();
         let mut i: usize = 0;
 
-        while read_byte_record_up_to(&mut rdr, &mut record, self.flag_end_byte)? {
+        while rdr.read_byte_record(&mut record)? {
             if indices.contains(&i) {
                 wtr.write_byte_record(&record)?;
             }
@@ -405,7 +451,7 @@ impl Args {
         Config::new(&self.flag_output)
     }
 
-    fn conditions(&self, headers: &csv::ByteRecord) -> CliResult<Conditions> {
+    fn conditions(&self, headers: &simd_csv::ByteRecord) -> CliResult<Conditions> {
         let start_condition_program = self
             .flag_start_condition
             .as_ref()
