@@ -1,5 +1,6 @@
 #[macro_use]
 extern crate serde_derive;
+extern crate fast_float2 as fast_float;
 
 use std::borrow::ToOwned;
 use std::env;
@@ -19,6 +20,7 @@ mod json;
 mod moonblade;
 mod ratatui;
 mod read;
+mod record;
 mod scales;
 mod select;
 mod urls;
@@ -47,6 +49,7 @@ macro_rules! command_list {
 
 ## Search & filter
     search      Search for (or replace) patterns in CSV data
+    grep        Coarse but fast filtering of CSV data
     filter      Only keep some CSV rows based on an evaluated expression
     head        First rows of CSV file
     tail        Last rows of CSV file
@@ -82,6 +85,7 @@ macro_rules! command_list {
     flatmap     Emit one row per value yielded by an expression evaluated for each CSV row
     fill        Fill empty cells
     blank       Blank down contiguous identical cell values
+    separate    Split a single column into multiple ones
 
 ## Format, convert & recombobulate
     behead        Drop header from CSV file
@@ -95,9 +99,11 @@ macro_rules! command_list {
     to            Convert a CSV file to a variety of data formats
     scrape        Scrape HTML into CSV data
     reverse       Reverse rows of CSV data
-    transpose (t) Transpose CSV file
-    pivot         Stack multiple columns into fewer columns
-    unpivot       Split distinct values of a column into their own columns
+
+## Transpose & pivot
+    transpose (t)            Transpose CSV file
+    pivot     (pivot-wider)  Stack multiple columns into fewer columns
+    unpivot   (pivot-longer) Split distinct values of a column into their own columns
 
 ## Split a CSV file into multiple
     split       Split CSV data into chunks
@@ -138,8 +144,7 @@ Options:
 Common options:
     -n, --no-headers       Typically used to indicate that input file has no headers.
     -d, --delimiter <arg>  Typically used to indicate a custom delimiter.
-
-Commands:",
+",
     command_list!()
 );
 
@@ -170,14 +175,14 @@ fn main() {
             match e {
                 docopt::Error::Deserialize(_) => {
                     // Command mismatch
-                    eprintln!(
-                        "Please choose one of the following commands/flags:\n{}",
-                        util::colorize_main_help(command_list!())
-                    );
+                    // eprintln!(
+                    //     "Please choose one of the following commands/flags:\n{}",
+                    //     util::colorize_main_help(command_list!())
+                    // );
                     eprintln!(
                         "{}",
                         format!(
-                            "Unknown command {}!\nUse one of the commands listed above.",
+                            "xan: unknown command \"{}\"! Use xan --help to review available commands.",
                             std::env::args()
                                 .nth(1)
                                 .unwrap_or_else(|| "<missing>".to_string())
@@ -221,6 +226,14 @@ Please choose one of the following commands/flags:\n{}",
                 );
                 process::exit(1);
             }
+            Err(CliError::SimdCsv(err)) => {
+                eprintln!(
+                    "xan {}: {}",
+                    env::args().nth(1).unwrap_or("".to_string()),
+                    err
+                );
+                process::exit(1);
+            }
             Err(CliError::Io(ref err)) if err.kind() == io::ErrorKind::BrokenPipe => {
                 process::exit(0);
             }
@@ -241,7 +254,11 @@ Please choose one of the following commands/flags:\n{}",
                 process::exit(1);
             }
             Err(CliError::Help(usage, exit_code)) => {
-                println!("{}", usage);
+                if exit_code == 0 {
+                    println!("{}", usage);
+                } else {
+                    eprintln!("{}", usage);
+                }
                 process::exit(exit_code);
             }
         },
@@ -277,6 +294,7 @@ enum Command {
     From,
     #[serde(rename = "fuzzy-join")]
     FuzzyJoin,
+    Grep,
     Groupby,
     Guillotine,
     H,
@@ -296,6 +314,10 @@ enum Command {
     Parallel,
     Partition,
     Pivot,
+    #[serde(rename = "pivot-longer")]
+    PivotLonger,
+    #[serde(rename = "pivot-wider")]
+    PivotWider,
     Plot,
     Progress,
     Range,
@@ -305,6 +327,7 @@ enum Command {
     Scrape,
     Search,
     Select,
+    Separate,
     Shuffle,
     Slice,
     Sort,
@@ -357,6 +380,7 @@ impl Command {
             Command::Freq | Command::Frequency => cmd::frequency::run(argv),
             Command::From => cmd::from::run(argv),
             Command::FuzzyJoin => cmd::fuzzy_join::run(argv),
+            Command::Grep => cmd::grep::run(argv),
             Command::Groupby => cmd::groupby::run(argv),
             Command::Head => cmd::head::run(argv),
             Command::Headers | Command::H => cmd::headers::run(argv),
@@ -379,7 +403,7 @@ impl Command {
             Command::Merge => cmd::merge::run(argv),
             Command::Parallel | Command::P => cmd::parallel::run(argv),
             Command::Partition => cmd::partition::run(argv),
-            Command::Pivot => cmd::pivot::run(argv),
+            Command::Pivot | Command::PivotWider => cmd::pivot::run(argv),
             Command::Plot => cmd::plot::run(argv),
             Command::Progress => cmd::progress::run(argv),
             Command::Range => cmd::range::run(argv),
@@ -389,6 +413,7 @@ impl Command {
             Command::Scrape => cmd::scrape::run(argv),
             Command::Search => cmd::search::run(argv),
             Command::Select => cmd::select::run(argv),
+            Command::Separate => cmd::separate::run(argv),
             Command::Shuffle => cmd::shuffle::run(argv),
             Command::Slice => cmd::slice::run(argv),
             Command::Sort => cmd::sort::run(argv),
@@ -400,7 +425,7 @@ impl Command {
             Command::Top => cmd::top::run(argv),
             Command::Transform => cmd::transform::run(argv),
             Command::Transpose | Command::T => cmd::transpose::run(argv),
-            Command::Unpivot => cmd::unpivot::run(argv),
+            Command::Unpivot | Command::PivotLonger => cmd::unpivot::run(argv),
             Command::View | Command::V => cmd::view::run(argv),
             Command::Vocab => cmd::vocab::run(argv),
             Command::Window => cmd::window::run(argv),
@@ -414,6 +439,7 @@ pub type CliResult<T> = Result<T, CliError>;
 pub enum CliError {
     Flag(docopt::Error),
     Csv(csv::Error),
+    SimdCsv(simd_csv::Error),
     Io(io::Error),
     Other(String),
     Help(String, i32),
@@ -424,6 +450,7 @@ impl fmt::Display for CliError {
         match *self {
             CliError::Flag(ref e) => e.fmt(f),
             CliError::Csv(ref e) => e.fmt(f),
+            CliError::SimdCsv(ref e) => e.fmt(f),
             CliError::Io(ref e) => e.fmt(f),
             CliError::Other(ref s) => f.write_str(s),
             CliError::Help(ref s, _) => f.write_str(s),
@@ -433,23 +460,26 @@ impl fmt::Display for CliError {
 
 impl From<docopt::Error> for CliError {
     fn from(err: docopt::Error) -> CliError {
-        use colored::Colorize;
-
         match err {
             docopt::Error::WithProgramUsage(kind, usage) => {
                 let usage = util::colorize_help(&usage);
 
+                let format_error_msg = |msg: &str| -> String {
+                    format!(
+                        "{}\n\n{} Use the {} flag for more information.",
+                        usage,
+                        msg.red(),
+                        "-h/--help".cyan()
+                    )
+                };
+
                 match kind.as_ref() {
                     docopt::Error::Help => CliError::Help(usage, 0),
-                    _ => CliError::Help(
-                        format!(
-                            "{}\n\n{} Use the {} flag for more information.",
-                            usage,
-                            "Invalid command!".red(),
-                            "-h/--help".cyan()
-                        ),
-                        1,
-                    ),
+                    docopt::Error::NoMatch => {
+                        CliError::Help(format_error_msg("Invalid subcommand or arguments!"), 1)
+                    }
+                    docopt::Error::Argv(msg) => CliError::Help(format_error_msg(msg), 1),
+                    _ => CliError::Help(format_error_msg("Invalid arguments!"), 1),
                 }
             }
             _ => CliError::Flag(err),
@@ -469,6 +499,18 @@ impl From<csv::Error> for CliError {
     }
 }
 
+impl From<simd_csv::Error> for CliError {
+    fn from(err: simd_csv::Error) -> Self {
+        if !err.is_io_error() {
+            return CliError::SimdCsv(err);
+        }
+        match err.into_kind() {
+            simd_csv::ErrorKind::Io(v) => From::from(v),
+            _ => unreachable!(),
+        }
+    }
+}
+
 impl From<io::Error> for CliError {
     fn from(err: io::Error) -> CliError {
         CliError::Io(err)
@@ -568,6 +610,12 @@ impl From<serde_json::Error> for CliError {
     }
 }
 
+impl From<simd_json::Error> for CliError {
+    fn from(value: simd_json::Error) -> Self {
+        CliError::Other(value.to_string())
+    }
+}
+
 impl From<url::ParseError> for CliError {
     fn from(value: url::ParseError) -> Self {
         CliError::Other(value.to_string())
