@@ -1,11 +1,12 @@
 use std::collections::{BTreeMap, BTreeSet};
+use std::hash::Hash;
 use std::iter::once;
 use std::sync::Arc;
 
 use ahash::RandomState;
-use csv::ByteRecord;
 use indexmap::IndexMap;
 use jiff::{civil::DateTime, Unit};
+use simd_csv::ByteRecord;
 
 use super::aggregators::{
     AllAny, ApproxCardinality, ApproxQuantiles, ArgExtent, ArgTop, Count, CovarianceWelford, First,
@@ -838,6 +839,10 @@ fn get_function_arguments_parser(name: &str) -> Option<(FunctionArguments, Argum
     })
 }
 
+pub fn is_agg_fn_name(name: &str) -> bool {
+    get_function_arguments_parser(name).is_some()
+}
+
 #[derive(Debug, Clone)]
 enum ConcreteAggregationMethod {
     All,
@@ -906,7 +911,7 @@ impl ConcreteAggregationMethod {
 }
 
 #[derive(Debug)]
-struct ConcreteAggregation {
+pub struct ConcreteAggregation {
     agg_name: String,
     method: ConcreteAggregationMethod,
     expr: Option<ConcreteExpr>,
@@ -919,9 +924,9 @@ impl ConcreteAggregation {
     }
 }
 
-type ConcreteAggregations = Vec<ConcreteAggregation>;
+pub type ConcreteAggregations = Vec<ConcreteAggregation>;
 
-fn concretize_aggregations(
+pub fn concretize_aggregations(
     aggregations: Aggregations,
     headers: &ByteRecord,
 ) -> Result<ConcreteAggregations, ConcretizationError> {
@@ -1173,19 +1178,29 @@ pub struct AggregationProgram {
 }
 
 impl AggregationProgram {
-    pub fn parse(code: &str, headers: &ByteRecord) -> Result<Self, ConcretizationError> {
-        let concrete_aggregations = prepare(code, headers)?;
+    pub fn from_concrete_aggregations(
+        concrete_aggregations: ConcreteAggregations,
+        headers: &ByteRecord,
+    ) -> Self {
         let len = concrete_aggregations.len();
         let planner = ConcreteAggregationPlanner::from(concrete_aggregations);
         let aggregators = planner.instantiate_aggregators();
 
-        Ok(Self {
+        Self {
             planner,
             aggregators,
             len,
             headers_index: HeadersIndex::from_headers(headers),
             last_value: DynamicValue::empty_bytes(),
-        })
+        }
+    }
+
+    pub fn parse(code: &str, headers: &ByteRecord) -> Result<Self, ConcretizationError> {
+        let concrete_aggregations = prepare(code, headers)?;
+        Ok(Self::from_concrete_aggregations(
+            concrete_aggregations,
+            headers,
+        ))
     }
 
     pub fn has_single_expr(&self) -> bool {
@@ -1243,14 +1258,21 @@ impl AggregationProgram {
         self.planner.headers()
     }
 
-    pub fn finalize(&mut self, parallel: bool) -> Result<ByteRecord, SpecifiedEvaluationError> {
+    pub fn finalize_iter(
+        &mut self,
+        parallel: bool,
+    ) -> impl Iterator<Item = Result<DynamicValue, SpecifiedEvaluationError>> + '_ {
         for aggregator in self.aggregators.iter_mut() {
             aggregator.finalize(parallel);
         }
 
+        self.planner.results(&self.aggregators, &self.headers_index)
+    }
+
+    pub fn finalize(&mut self, parallel: bool) -> Result<ByteRecord, SpecifiedEvaluationError> {
         let mut record = ByteRecord::new();
 
-        for value in self.planner.results(&self.aggregators, &self.headers_index) {
+        for value in self.finalize_iter(parallel) {
             record.push_field(&value?.serialize_as_bytes());
         }
 
@@ -1258,27 +1280,44 @@ impl AggregationProgram {
     }
 }
 
-type GroupKey = Vec<Vec<u8>>;
-
 #[derive(Debug, Clone)]
-pub struct GroupAggregationProgram {
+pub struct GroupAggregationProgram<K> {
     planner: ConcreteAggregationPlanner,
-    groups: ClusteredInsertHashmap<GroupKey, Vec<CompositeAggregator>>,
+    groups: ClusteredInsertHashmap<K, Vec<CompositeAggregator>>,
     headers_index: HeadersIndex,
+    len: usize,
+    dummy_record: ByteRecord,
+    last_value: DynamicValue,
 }
 
-impl GroupAggregationProgram {
+impl<K: Eq + Hash> GroupAggregationProgram<K> {
     pub fn parse(code: &str, headers: &ByteRecord) -> Result<Self, ConcretizationError> {
         let concrete_aggregations = prepare(code, headers)?;
+        let len = concrete_aggregations.len();
         let planner = ConcreteAggregationPlanner::from(concrete_aggregations);
 
         Ok(Self {
             planner,
             groups: ClusteredInsertHashmap::new(),
             headers_index: HeadersIndex::from_headers(headers),
+            len,
+            dummy_record: ByteRecord::new(),
+            last_value: DynamicValue::empty_bytes(),
         })
     }
 
+    pub fn parse_without_headers(code: &str) -> Result<Self, ConcretizationError> {
+        Self::parse(code, &ByteRecord::new())
+    }
+
+    pub fn len(&self) -> usize {
+        self.len
+    }
+
+    pub fn has_single_expr(&self) -> bool {
+        self.len == 1
+    }
+
     pub fn merge(&mut self, other: Self) {
         for (key, other_aggregators) in other.groups.into_iter() {
             self.groups.insert_or_update_with(
@@ -1297,7 +1336,7 @@ impl GroupAggregationProgram {
 
     pub fn run_with_record(
         &mut self,
-        group: GroupKey,
+        group: K,
         index: usize,
         record: &ByteRecord,
     ) -> Result<(), SpecifiedEvaluationError> {
@@ -1317,14 +1356,86 @@ impl GroupAggregationProgram {
         )
     }
 
+    pub fn run_with_cells<'a>(
+        &mut self,
+        group: K,
+        index: usize,
+        record: &ByteRecord,
+        cells: impl Iterator<Item = &'a [u8]>,
+    ) -> Result<(), SpecifiedEvaluationError> {
+        let planner = &self.planner;
+
+        let aggregators = self
+            .groups
+            .insert_with(group, || planner.instantiate_aggregators());
+
+        for cell in cells {
+            self.last_value.set_bytes(cell);
+
+            run_with_record_on_aggregators(
+                &self.planner,
+                aggregators.iter_mut(),
+                index,
+                record,
+                &self.headers_index,
+                Some(self.last_value.clone()),
+            )?;
+        }
+
+        Ok(())
+    }
+
+    pub fn run_with<T: Into<DynamicValue>>(
+        &mut self,
+        group: K,
+        index: usize,
+        value: T,
+    ) -> Result<(), SpecifiedEvaluationError> {
+        let planner = &self.planner;
+
+        let aggregators = self
+            .groups
+            .insert_with(group, || planner.instantiate_aggregators());
+
+        run_with_record_on_aggregators(
+            &self.planner,
+            aggregators.iter_mut(),
+            index,
+            &self.dummy_record,
+            &self.headers_index,
+            Some(value.into()),
+        )
+    }
+
     pub fn headers(&self) -> impl Iterator<Item = &[u8]> {
         self.planner.headers()
     }
 
+    pub fn iter(self) -> impl Iterator<Item = Result<(K, DynamicValue), SpecifiedEvaluationError>> {
+        assert!(self.has_single_expr());
+
+        let planner = self.planner;
+        let headers_index = self.headers_index;
+
+        self.groups
+            .into_iter()
+            .map(move |(group, mut aggregators)| {
+                for aggregator in aggregators.iter_mut() {
+                    aggregator.finalize(false);
+                }
+
+                planner
+                    .results(&aggregators, &headers_index)
+                    .next()
+                    .unwrap()
+                    .map(|value| (group, value))
+            })
+    }
+
     pub fn into_byte_records(
         self,
         parallel: bool,
-    ) -> impl Iterator<Item = Result<(GroupKey, ByteRecord), SpecifiedEvaluationError>> {
+    ) -> impl Iterator<Item = Result<(K, ByteRecord), SpecifiedEvaluationError>> {
         let planner = self.planner;
         let headers_index = self.headers_index;
 
@@ -1348,7 +1459,7 @@ impl GroupAggregationProgram {
 
 #[derive(Debug, Clone, Default)]
 struct PivotedColumnNamesIndex {
-    names: Vec<Vec<u8>>,
+    names: ByteRecord,
     seen: BTreeSet<Vec<u8>>,
 }
 
@@ -1356,11 +1467,13 @@ impl PivotedColumnNamesIndex {
     fn add(&mut self, name: &[u8]) {
         if !self.seen.contains(name) {
             self.seen.insert(name.to_vec());
-            self.names.push(name.to_vec());
+            self.names.push_field(name);
         }
     }
 }
 
+type GroupKey = Vec<Vec<u8>>;
+
 #[derive(Debug, Clone)]
 pub struct PivotAggregationProgram {
     planner: ConcreteAggregationPlanner,
@@ -1430,16 +1543,16 @@ impl PivotAggregationProgram {
         )
     }
 
-    pub fn pivoted_column_names(&self) -> &[Vec<u8>] {
+    pub fn pivoted_column_names(&self) -> &ByteRecord {
         &self.pivoted_column_names_index.names
     }
 
     pub fn flush<F, E>(self, mut callback: F) -> Result<(), E>
     where
-        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        F: FnMut(&ByteRecord) -> Result<(), E>,
         E: From<SpecifiedEvaluationError>,
     {
-        let mut record = csv::ByteRecord::new();
+        let mut record = ByteRecord::new();
         let names = self.pivoted_column_names_index.names;
 
         for (group, mut pivot_map) in self.groups.into_iter() {
