@@ -25,7 +25,7 @@ use unicode_width::UnicodeWidthStr;
 
 use crate::config::{Config, Delimiter};
 use crate::dates;
-use crate::select::SelectColumns;
+use crate::select::SelectedColumns;
 use crate::CliResult;
 
 pub fn version() -> String {
@@ -47,10 +47,13 @@ pub fn version() -> String {
     }
 }
 
+pub fn default_num_cpus() -> usize {
+    num_cpus::get().min(16)
+}
+
 lazy_static! {
-    static ref FLAG_REGEX: Regex = Regex::new(r"([\s,/\(])(--?[A-Za-z§][\w\-]*)").unwrap();
-    static ref SECTION_REGEX: Regex =
-        Regex::new("(?im)^.*(?:usage|options|commands?):|---+").unwrap();
+    static ref FLAG_REGEX: Regex = Regex::new(r"([\s,/\(])(--?[A-Za-z§][\w\-=]*)").unwrap();
+    static ref SECTION_REGEX: Regex = Regex::new("(?im)^.*(?:usage|options):|---+").unwrap();
     static ref DIMMED_REGEX: Regex = Regex::new(
         r"\[--\]|\[?<[\w|\-]+>(?:\.{3})?\]?|\[[\w\s:§|\-.]+\]|\s+[\$>][^\n]+|\*[^*\n]+\*"
     )
@@ -92,6 +95,24 @@ pub fn colorize_main_help(help: &str) -> String {
     help.replace("xan", &"xan".red().to_string())
 }
 
+#[derive(Debug, Clone, Copy, Default, Deserialize)]
+pub enum ColorMode {
+    #[default]
+    Auto,
+    Never,
+    Always,
+}
+
+impl ColorMode {
+    pub fn apply(&self) {
+        match self {
+            Self::Never => colored::control::set_override(false),
+            Self::Always => colored::control::set_override(true),
+            _ => (),
+        }
+    }
+}
+
 pub fn get_args<T>(usage: &str, argv: &[&str]) -> CliResult<T>
 where
     T: DeserializeOwned,
@@ -110,7 +131,7 @@ pub fn many_configs(
     inps: &[String],
     delim: Option<Delimiter>,
     no_headers: bool,
-    select: Option<&SelectColumns>,
+    select: Option<&SelectedColumns>,
 ) -> Result<Vec<Config>, String> {
     let mut inps = inps.to_vec();
     if inps.is_empty() {
@@ -195,7 +216,7 @@ impl FilenameTemplate {
         &self,
         path: P,
         unique_value: &str,
-    ) -> io::Result<csv::Writer<Box<dyn io::Write + 'static>>>
+    ) -> io::Result<simd_csv::Writer<Box<dyn io::Write + 'static>>>
     where
         P: AsRef<Path>,
     {
@@ -208,7 +229,7 @@ impl FilenameTemplate {
             fs::create_dir_all(parent)?;
         }
         let spath = Some(full_path.display().to_string());
-        Config::new(&spath).writer_with_options(
+        Config::new(&spath).simd_writer_with_options(
             fs::OpenOptions::new()
                 .write(true)
                 .create(true)
@@ -369,8 +390,17 @@ pub enum ColorOrStyles {
 }
 
 impl ColorOrStyles {
-    pub fn is_green(&self) -> bool {
-        matches!(self, Self::Color(color) if color == &Color::Green)
+    pub fn highlightable_color(&self) -> Option<Cow<'static, str>> {
+        match self {
+            Self::Color(color) => {
+                if color != &Color::Red {
+                    Some(color.to_fg_str())
+                } else {
+                    None
+                }
+            }
+            _ => None,
+        }
     }
 }
 
@@ -380,7 +410,7 @@ pub fn colorizer_by_type(string: &str) -> ColorOrStyles {
             return ColorOrStyles::Color(Color::Cyan)
         }
         "NULL" | "null" | "na" | "NA" | "None" | "n/a" | "N/A" | "nan" | "NaN" | "<empty>"
-        | "<null>" | "<rest>" | "." | "-" => return ColorOrStyles::Styles(Styles::Dimmed),
+        | "<rest>" | "." | "-" => return ColorOrStyles::Styles(Styles::Dimmed),
         _ => (),
     };
 
@@ -427,20 +457,32 @@ pub fn colorize(color_or_style: &ColorOrStyles, string: &str) -> ColoredString {
     }
 }
 
-pub fn highlight_trimmable_whitespace(string: &str) -> String {
-    let start = string.len() - string.trim_start().len();
-    let end = string.trim_end().len();
-
-    format!(
-        "{}{}{}",
-        "·".repeat((0..start).len()).white().dimmed(),
-        &string[start..end],
-        "·".repeat((end..string.len()).len()).white().dimmed()
-    )
+lazy_static! {
+    static ref ESCAPED_WHITESPACE_REPLACER: Regex = Regex::new(r"\\[nrtf]").unwrap();
 }
 
-lazy_static! {
-    static ref WHITESPACE_REPLACER: Regex = Regex::new(r"\r\n|\n\r|[\n\r\t\f]").unwrap();
+pub fn highlight_problematic_string_features(string: &str) -> String {
+    let start = string.len() - string.trim_start().len();
+    let end = start + string[start..].trim_end().len();
+
+    let replaced = format!(
+        "{}{}{}",
+        if (0..start).is_empty() {
+            "".normal()
+        } else {
+            "·".repeat((0..start).len()).white().dimmed()
+        },
+        &string[start..end],
+        if (end..string.len()).is_empty() {
+            "".normal()
+        } else {
+            "·".repeat((end..string.len()).len()).white().dimmed()
+        }
+    );
+
+    ESCAPED_WHITESPACE_REPLACER
+        .replace_all(&replaced, |caps: &Captures| caps[0].dimmed().to_string())
+        .into_owned()
 }
 
 pub fn sanitize_text_for_multi_line_printing(string: &str) -> String {
@@ -453,10 +495,21 @@ pub fn sanitize_text_for_multi_line_printing(string: &str) -> String {
     string
 }
 
+lazy_static! {
+    static ref WHITESPACE_REPLACER: Regex = Regex::new(r"\r\n|[\n\r\t\f]").unwrap();
+}
+
 pub fn sanitize_text_for_single_line_printing(string: &str) -> String {
     let sanitized = sanitize_text_for_multi_line_printing(string);
 
-    match WHITESPACE_REPLACER.replace_all(&sanitized, " ") {
+    match WHITESPACE_REPLACER.replace_all(&sanitized, |caps: &Captures| match &caps[0] {
+        "\r\n" => "\\r\\n",
+        "\n" => "\\n",
+        "\r" => "\\r",
+        "\t" => "\\t",
+        "\x0C" => "\\f",
+        _ => unreachable!(),
+    }) {
         Cow::Borrowed(_) => sanitized,
         Cow::Owned(s) => s,
     }
@@ -580,10 +633,12 @@ pub fn unicode_aware_highlighted_pad_with_ellipsis(
     padding: &str,
     highlight: bool,
 ) -> String {
+    let with_ellipsis = unicode_aware_ellipsis(string, width);
+
     let mut string = unicode_aware_pad(
         left,
         &(if highlight {
-            highlight_trimmable_whitespace(&unicode_aware_ellipsis(string, width))
+            highlight_problematic_string_features(&with_ellipsis)
         } else {
             unicode_aware_ellipsis(string, width)
         }),
@@ -660,86 +715,6 @@ impl EmojiSanitizer {
     }
 }
 
-pub trait ImmutableRecordHelpers<'a> {
-    type Cell;
-
-    #[must_use]
-    fn replace_at(&self, column_index: usize, new_value: Self::Cell) -> Self;
-
-    #[must_use]
-    fn prepend(&self, cell_value: Self::Cell) -> Self;
-
-    #[must_use]
-    fn append(&self, cell_value: Self::Cell) -> Self;
-
-    #[must_use]
-    fn remove(&self, column_index: usize) -> Self;
-}
-
-impl<'a> ImmutableRecordHelpers<'a> for csv::ByteRecord {
-    type Cell = &'a [u8];
-
-    fn replace_at(&self, column_index: usize, new_value: Self::Cell) -> Self {
-        self.iter()
-            .enumerate()
-            .map(|(i, v)| if i == column_index { new_value } else { v })
-            .collect()
-    }
-
-    fn prepend(&self, cell_value: Self::Cell) -> Self {
-        let mut new_record = csv::ByteRecord::new();
-        new_record.push_field(cell_value);
-        new_record.extend(self);
-
-        new_record
-    }
-
-    fn append(&self, cell_value: Self::Cell) -> Self {
-        let mut new_record = self.clone();
-        new_record.push_field(cell_value);
-        new_record
-    }
-
-    fn remove(&self, column_index: usize) -> Self {
-        self.iter()
-            .enumerate()
-            .filter_map(|(i, c)| if i == column_index { None } else { Some(c) })
-            .collect()
-    }
-}
-
-impl<'a> ImmutableRecordHelpers<'a> for csv::StringRecord {
-    type Cell = &'a str;
-
-    fn replace_at(&self, column_index: usize, new_value: Self::Cell) -> Self {
-        self.iter()
-            .enumerate()
-            .map(|(i, v)| if i == column_index { new_value } else { v })
-            .collect()
-    }
-
-    fn prepend(&self, cell_value: Self::Cell) -> Self {
-        let mut new_record = csv::StringRecord::new();
-        new_record.push_field(cell_value);
-        new_record.extend(self);
-
-        new_record
-    }
-
-    fn append(&self, cell_value: Self::Cell) -> Self {
-        let mut new_record = self.clone();
-        new_record.push_field(cell_value);
-        new_record
-    }
-
-    fn remove(&self, column_index: usize) -> Self {
-        self.iter()
-            .enumerate()
-            .filter_map(|(i, c)| if i == column_index { None } else { Some(c) })
-            .collect()
-    }
-}
-
 pub fn str_to_csv_byte_record(target: &str) -> csv::ByteRecord {
     let cursor = io::Cursor::new(target);
     let reader = csv::ReaderBuilder::new()
@@ -819,21 +794,21 @@ impl<T: Deref<Target = str>, I: Iterator<Item = T>> JoinIteratorExt for I {
 }
 
 // A custom implementation de/serializing ext-sort chunks as CSV
-pub struct DeepSizedByteRecord(pub csv::ByteRecord);
+pub struct DeepSizedByteRecord(pub simd_csv::ByteRecord);
 
 impl DeepSizedByteRecord {
-    pub fn as_ref(&self) -> &csv::ByteRecord {
+    pub fn as_ref(&self) -> &simd_csv::ByteRecord {
         &self.0
     }
 
-    pub fn into_inner(self) -> csv::ByteRecord {
+    pub fn into_inner(self) -> simd_csv::ByteRecord {
         self.0
     }
 }
 
 impl DeepSizeOf for DeepSizedByteRecord {
     fn deep_size_of(&self) -> usize {
-        std::mem::size_of::<csv::ByteRecord>() + self.0.as_slice().len()
+        std::mem::size_of::<simd_csv::ByteRecord>() + self.0.as_slice().len()
     }
 
     fn deep_size_of_children(&self, _context: &mut deepsize::Context) -> usize {
@@ -842,16 +817,16 @@ impl DeepSizeOf for DeepSizedByteRecord {
 }
 
 pub struct CsvExternalChunk {
-    reader: csv::Reader<io::Take<io::BufReader<fs::File>>>,
+    reader: simd_csv::Reader<io::Take<io::BufReader<fs::File>>>,
 }
 
 impl ExternalChunk<DeepSizedByteRecord> for CsvExternalChunk {
-    type SerializationError = csv::Error;
-    type DeserializationError = csv::Error;
+    type SerializationError = simd_csv::Error;
+    type DeserializationError = simd_csv::Error;
 
     fn new(reader: io::Take<io::BufReader<fs::File>>) -> Self {
         CsvExternalChunk {
-            reader: csv::ReaderBuilder::new()
+            reader: simd_csv::ReaderBuilder::new()
                 .has_headers(false)
                 .from_reader(reader),
         }
@@ -861,7 +836,7 @@ impl ExternalChunk<DeepSizedByteRecord> for CsvExternalChunk {
         chunk_writer: &mut io::BufWriter<fs::File>,
         items: impl IntoIterator<Item = DeepSizedByteRecord>,
     ) -> Result<(), Self::SerializationError> {
-        let mut csv_writer = csv::Writer::from_writer(chunk_writer);
+        let mut csv_writer = simd_csv::Writer::from_writer(chunk_writer);
 
         for item in items.into_iter() {
             csv_writer.write_record(item.as_ref())?;
@@ -878,7 +853,7 @@ impl Iterator for CsvExternalChunk {
     >;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let mut record = csv::ByteRecord::new();
+        let mut record = simd_csv::ByteRecord::new();
 
         match self.reader.read_byte_record(&mut record) {
             Ok(read) => {
