@@ -28,6 +28,7 @@ mod test_flatmap;
 mod test_fmt;
 mod test_frequency;
 mod test_fuzzy_join;
+mod test_grep;
 mod test_groupby;
 mod test_headers;
 mod test_implode;
@@ -44,6 +45,7 @@ mod test_sample;
 mod test_scrape;
 mod test_search;
 mod test_select;
+mod test_separate;
 mod test_shuffle;
 mod test_slice;
 mod test_sort;
